@@ -3,7 +3,7 @@
  * The main Horde_Ldap class.
  *
  * Copyright 2003-2007 Tarjej Huse, Jan Wagner, Del Elson, Benedikt Hallinger
- * Copyright 2009-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2017 Horde LLC (http://www.horde.org/)
  *
  * @package   Ldap
  * @author    Tarjej Huse <tarjei@bergfald.no>
@@ -18,33 +18,33 @@
 class Horde_Ldap
 {
     /**
-     * Class configuration array
+     * Class configuration array.
      *
-     * - hostspec:       the LDAP host to connect to (may be an array of
+     * - hostspec:       The LDAP host to connect to (may be an array of
      *                   several hosts to try).
-     * - port:           the server port.
+     * - port:           The server port.
      * - version:        LDAP version (defaults to 3).
-     * - tls:            when set, ldap_start_tls() is run after connecting.
-     * - binddn:         the DN to bind as when searching.
-     * - bindpw:         password to use when searching LDAP.
+     * - tls:            When set, ldap_start_tls() is run after connecting.
+     * - binddn:         The DN to bind as when searching.
+     * - bindpw:         Password to use when searching LDAP.
      * - basedn:         LDAP base.
-     * - options:        hash of LDAP options to set.
-     * - filter:         default search filter.
-     * - scope:          default search scope.
-     * - user:           configuration parameters for {@link findUserDN()},
+     * - options:        Hash of LDAP options to set.
+     * - filter:         Default search filter.
+     * - scope:          Default search scope.
+     * - user:           Configuration parameters for {@link findUserDN()},
      *                   must contain 'uid', and may contain 'basedn'
      *                   entries.
      * - timeout:        Connection timeout in seconds (defaults to 5).
-     * - auto_reconnect: if true, the class will automatically
+     * - auto_reconnect: If true, the class will automatically
      *                   attempt to reconnect to the LDAP server in certain
      *                   failure conditions when attempting a search, or other
      *                   LDAP operations.  Defaults to false.  Note that if you
      *                   set this to true, calls to search() may block
      *                   indefinitely if there is a catastrophic server failure.
-     * - min_backoff:    minimum reconnection delay period (in seconds).
-     * - current_backof: initial reconnection delay period (in seconds).
-     * - max_backoff:    maximum reconnection delay period (in seconds).
-     * - cache           a Horde_Cache instance for caching schema requests.
+     * - min_backoff:    Minimum reconnection delay period (in seconds).
+     * - current_backof: Initial reconnection delay period (in seconds).
+     * - max_backoff:    Maximum reconnection delay period (in seconds).
+     * - cache:          A Horde_Cache instance for caching schema requests.
      *
      * @var array
      */
@@ -66,6 +66,7 @@ class Horde_Ldap
         'current_backoff' => 1,
         'max_backoff'     => 32,
         'cache'           => false,
+        'cache_root_dse'  => false,
         'cachettl'        => 3600);
 
     /**
@@ -97,14 +98,6 @@ class Horde_Ldap
      */
     protected $_schema;
 
-    /**
-     * Schema cache function callback.
-     *
-     * @see registerSchemaCache()
-     * @var string
-     */
-    protected $_schemaCache;
-
     /**
      * Cache for attribute encoding checks.
      *
@@ -114,7 +107,7 @@ class Horde_Ldap
     protected $_schemaAttrs = array();
 
     /**
-     * Cache for rootDSE objects
+     * Cache for rootDSE objects.
      *
      * Hash with requested rootDSE attr names as key and rootDSE
      * object as value.
@@ -126,7 +119,7 @@ class Horde_Ldap
      *
      * @var array
      */
-    protected $_rootDSECache = array();
+    protected $_rootDSE = array();
 
     /**
      * Constructor.
@@ -1289,15 +1282,39 @@ class Horde_Ldap
      */
     public function rootDSE(array $attrs = array())
     {
-        $attrs_signature = serialize($attrs);
+        /* If a cache object is registered, we use that to fetch a rootDSE
+         * object. */
+        $key = 'Horde_Ldap_RootDse_'
+            . md5(serialize(array(
+                $this->_config['hostspec'], $this->_config['port'], $attrs
+            )));
+        if (empty($this->_rootDSE[$key]) &&
+            $this->_config['cache'] &&
+            $this->_config['cache_root_dse']) {
+            $entry = $this->_config['cache']->get(
+                $key, $this->_config['cachettl']
+            );
+            if ($entry) {
+                $this->_rootDSE[$key] = @unserialize($entry);
+            }
+        }
 
         /* See if we need to fetch a fresh object, or if we already
          * requested this object with the same attributes. */
-        if (!isset($this->_rootDSECache[$attrs_signature])) {
-            $this->_rootDSECache[$attrs_signature] = new Horde_Ldap_RootDse($this, $attrs);
+        if (empty($this->_rootDSE[$key])) {
+            $this->_rootDSE[$key] = new Horde_Ldap_RootDse($this, $attrs);
+
+            /* If caching is active, advise the cache to store the object. */
+            if ($this->_config['cache'] && $this->_config['cache_root_dse']) {
+                $this->_config['cache']->set(
+                    $key,
+                    serialize($this->_rootDSE[$key]),
+                    $this->_config['cachettl']
+                );
+            }
         }
 
-        return $this->_rootDSECache[$attrs_signature];
+        return $this->_rootDSE[$key];
     }
 
     /**
